/**
 * Extend the basic ActorSheet with some very simple modifications
 * @extends {ActorSheet}
 */

import { TeDeumUtility } from "../common/tedeum-utility.js";

/* -------------------------------------------- */
export class TeDeumActorPJSheet extends foundry.appv1.sheets.ActorSheet {

  /** @override */
  static get defaultOptions() {

    return foundry.utils.mergeObject(super.defaultOptions, {
      classes: ["fvtt-te-deum", "sheet", "actor"],
      template: "systems/fvtt-te-deum/templates/actors/actor-sheet.hbs",
      width: 860,
      height:680,
      tabs: [{ navSelector: ".sheet-tabs", contentSelector: ".sheet-body", initial: "skills" }],
      dragDrop: [{ dragSelector: ".item-list .item", dropSelector: null }],
      editScore: true
    });
  }

  /* -------------------------------------------- */
  async getData() {

    let formData = {
      title: this.title,
      id: this.actor.id,
      type: this.actor.type,
      img: this.actor.img,
      name: this.actor.name,
      editable: this.isEditable,
      cssClass: this.isEditable ? "editable" : "locked",
      system: foundry.utils.duplicate(this.object.system),
      limited: this.object.limited,
      competences: this.actor.getCompetences(),
      config: foundry.utils.duplicate(game.system.tedeum.config),
      armes: this.actor.getArmes(),
      caracList: this.actor.prepareCaracteristiques(),
      providence: this.actor.prepareProvidence(),
      arbreCompetences: this.actor.prepareArbreCompetences(),
      equipements: this.actor.getEquipements(),
      simples: this.actor.getSimples(),
      armures: this.actor.getArmures(),
      graces: this.actor.getGraces(),
      blessures: this.actor.getBlessures(),
      maladies: this.actor.getMaladies(),
      poisons: this.actor.getPoisons(),
      combat: this.actor.prepareCombat(),
      bonusDegats: this.actor.getBonusDegats(),
      nbActions: this.actor.getNbActions(),
      initiative: this.actor.getInitiative(),
      pointsArmuresLourdes: this.actor.getNbArmures(),
      nbArmuresLourdes: this.actor.getNbArmuresLourdesActuel(),
      santeModifier: this.actor.getSanteModifier(),
      educations: this.actor.getEducations(),
      description: await foundry.applications.ux.TextEditor.implementation.enrichHTML(this.object.system.description, { async: true }),
      equipmentfree: await foundry.applications.ux.TextEditor.implementation.enrichHTML(this.object.system.equipmentfree, { async: true }),
      notes: await foundry.applications.ux.TextEditor.implementation.enrichHTML(this.object.system.notes, { async: true }),
      histoire: await foundry.applications.ux.TextEditor.implementation.enrichHTML(this.object.system.histoire, { async: true }),
      options: this.options,
      owner: this.document.isOwner,
      editScore: this.options.editScore,
      isGM: game.user.isGM
    }
    this.formData = formData;

    console.log("PC : ", formData, this.object);
    return formData;
  }


  /* -------------------------------------------- */
  /** @override */
  activateListeners(html) {
    super.activateListeners(html);

    // Everything below here is only needed if the sheet is editable
    if (!this.options.editable) return;

    html.bind("keydown", function(e) { // Ignore Enter in actores sheet
      if (e.keyCode === 13) return false;
    });

    // Update Inventory Item
    html.find('.item-edit').click(ev => {
      const li = $(ev.currentTarget).parents(".item-id")
      let itemId = li.data("item-id")
      const item = this.actor.items.get( itemId );
      item.sheet.render(true);
    });
    // Delete Inventory Item
    html.find('.item-delete').click(ev => {
      const li = $(ev.currentTarget).parents(".item-id")
      TeDeumUtility.confirmDelete(this, li).catch("Error : No deletion confirmed")
    })
    html.find('.item-add').click(ev => {
      let dataType = $(ev.currentTarget).data("type")
      this.actor.createEmbeddedDocuments('Item', [{ name: "Nouveau " + dataType, type: dataType }], { renderSheet: true })
    })
    html.find('.competence-add').click(ev => {
      let dataType = $(ev.currentTarget).data("type")
      let caracKey = $(ev.currentTarget).data("carac-key")
      this.actor.createEmbeddedDocuments('Item', [{ name: "Nouvelle " + dataType, type: dataType, system: {caracteristique: caracKey} }], { renderSheet: true })
    })

    html.find('.subactor-edit').click(ev => {
      const li = $(ev.currentTarget).parents(".item");
      let actorId = li.data("actor-id");
      let actor = game.actors.get( actorId );
      actor.sheet.render(true);
    });

    html.find('.subactor-delete').click(ev => {
      const li = $(ev.currentTarget).parents(".item");
      let actorId = li.data("actor-id");
      this.actor.delSubActor(actorId);
    });
    html.find('.quantity-minus').click(event => {
      const li = $(event.currentTarget).parents(".item");
      this.actor.incDecQuantity( li.data("item-id"), -1 );
    } );
    html.find('.quantity-plus').click(event => {
      const li = $(event.currentTarget).parents(".item");
      this.actor.incDecQuantity( li.data("item-id"), +1 );
    } );

    html.find('.roll-competence').click((event) => {
      let compId = $(event.currentTarget).data("comp-id")
      this.actor.rollCompetence(compId)
    });
    html.find('.roll-arme').click((event) => {
      const armeId = $(event.currentTarget).data("arme-id")
      this.actor.rollArme(armeId)
    });
    html.find('.roll-degats').click((event) => {
      const armeId = $(event.currentTarget).data("arme-id")
      this.actor.rollDegatsArme(armeId)
    });


    html.find('.lock-unlock-sheet').click((event) => {
      this.options.editScore = !this.options.editScore;
      this.render(true);
    });
    html.find('.item-equip').click(ev => {
      const li = $(ev.currentTarget).parents(".item");
      this.actor.equipItem( li.data("item-id") );
      this.render(true);
    });
    html.find('.update-field').change(ev => {
      const fieldName = $(ev.currentTarget).data("field-name");
      let value = Number(ev.currentTarget.value);
      this.actor.update( { [`${fieldName}`]: value } );
    });
  }

  /* -------------------------------------------- */
  /** @override */
  setPosition(options = {}) {
    const position = super.setPosition(options);
    const sheetBody = this.element.find(".sheet-body");
    const bodyHeight = position.height - 192;
    sheetBody.css("height", bodyHeight);
    return position;
  }

  /* -------------------------------------------- */
  /** @override */
  _updateObject(event, formData) {
    // Update the Actor
    return this.object.update(formData);
  }
}
